// --------------------------------------
// I2C_PICDHTtest
// --------------------------------------
//
// просто считывает влажность и температуру и периодически их отображает
// для всех трех устройств DHT22, подключенных к PIC
//
#include <Wire.h>
// глобальные переменные
byte DHT[16];  // для хранения/доступа к байтовому массиву в PIC
// адрес I2C (установлен в коде для PIC)
int address=0x19;
// хранение влажности и температуры
float TempCz1, TempFz1, RHz1, TempCz2, TempFz2, RHz2, TempCz3, TempFz3, RHz3;

void setup()
{
  Wire.begin();    // подключиться к шине в качестве мастера
  Serial.begin(9600);
  Serial.println("PIC Triple DHT22 I2C Humidity and Temperature");
  resetPIC();      // возможно, это необязательно, но хуже не будет
  delay(3000);     // чтобы дать мне время открыть монитор последовательного порта
}  
 
void loop()
{
  get_DHT1();  
  get_DHT2();
  get_DHT3(); 
  Serial.println();
  delay(5000);    // каждые 5 секунд

}

void get_DHT1()
{
  // DHT[0]=байт команды - установить мастером в значение 2 для запроса показаний DHT #1
  //    и при возврате он будет установлен в 1 в случае успеха.
  // 5 байтов информации DHT означают следующее:
  // данные DHT
  // DHT[1] влажность, целая часть
  // DHT[2] влажность, часть после запятой
  // DHT[3] температура, целая часть
  // DHT[4] температура, часть после запятой
  // DHT[5} контрольная сумма – это суммпа всех четырех байтов AND 255
  Wire.beginTransmission(address);
  Wire.write(0); // индекс в буфере
  Wire.write(2); // байт команды (2=прочитать DHT22 #1)
  Wire.endTransmission();
  delay(50);     // ждать выполнения - для начала 50 миллисекунд
  Wire.beginTransmission(address);
  Wire.write(0); // индекс в буфере
  Wire.endTransmission();
  Wire.requestFrom(address, 6);
  // скинуть первые 6 байт в массив
  for (int count=0;count<6; count++)
  {
      DHT[count]=Wire.read();
  }
  Serial.println();  
  // проверить, указывает ли байт команды, что всё прошло хорошо 
  if (DHT[0]!=1)
  {
    Serial.println("*** Read Error ***");
    // далее обрабатываем ошибку, если это необходимо
  } 
  // Влажность
  RHz1=DHT[1];
  RHz1*=256;
  RHz1+=DHT[2];
  RHz1/=10;
  // Температура  
  TempCz1=DHT[3]&0x7f;
  TempCz1*=256;
  TempCz1+=DHT[4];
  TempCz1/=10;
  if(DHT[3]&0x80)
  {
    TempCz1*=-1;
  }
  TempFz1=TempCz1 * 9 / 5 +32; 
  
  // контрольная сумма
  if ((DHT[1]+DHT[2]+DHT[3]+DHT[4] & 255) != DHT[5])
  {
    Serial.print(" *** BAD CRC! ***  ");  
  } 
  else
  {
    Serial.print(" CRC is good  ");
  }
  Serial.print("ZONE 1 Temperature ");
  Serial.print(TempCz1);
  Serial.print(" (C)  ");
  Serial.print(TempFz1);
  Serial.print(" (F) ");
  Serial.print("  Humidity ");
  Serial.println(RHz1); 
}

void get_DHT2()
{
  // DHT[0]=байт команды - установить мастером в значение 3 для запроса показаний DHT #2
  //    и при возврате он будет установлен в 1 в случае успеха или в значение ошибки.
  // 5 байтов информации DHT означают следующее:
  // данные DHT
  // DHT[6] влажность, целая часть
  // DHT[7] влажность, часть после запятой
  // DHT[8] температура, целая часть
  // DHT[9] температура, часть после запятой
  // DHT[10} контрольная сумма – это суммпа всех четырех байтов AND 255  
  Wire.beginTransmission(address);
  Wire.write(0); // индекс в буфере
  Wire.write(3); // байт команды (3=прочитать DHT22 #2) 
  Wire.endTransmission();
  delay(50);     // ждать выполнения - для начала 50 миллисекунд
  Wire.beginTransmission(address);
  Wire.write(0); // индекс в буфере
  Wire.endTransmission();
  Wire.requestFrom(address, 1);
  // обновить байт статуса
  DHT[0]=Wire.read();
  // теперь получим 5 байтов данных в массив
  Wire.beginTransmission(address);
  Wire.write(6); // индекс в буфере (указывает на данные для DHT #2)
  Wire.endTransmission();
  Wire.requestFrom(address, 5);
  for (int count=6;count<11; count++)
  {
    DHT[count]=Wire.read();
  } 
  // проверить, указывает ли байт команды, что всё прошло хорошо 
  if (DHT[0]!=1)
  {
    Serial.println("*** Read Error ***");
    // далее обрабатываем ошибку, если это необходимо
  }  
  // Влажность
  RHz2=DHT[6];
  RHz2*=256;
  RHz2+=DHT[7];
  RHz2/=10;
  // Температура  
  TempCz2=DHT[8]&0x7f;
  TempCz2*=256;
  TempCz2+=DHT[9];
  TempCz2/=10;
  if(DHT[8]&0x80)
  {
    TempCz2*=-1;
  }
  TempFz2=TempCz2 * 9 / 5 +32;   
  
  // контрольная сумма
  if ((DHT[6]+DHT[7]+DHT[8]+DHT[9] & 255) != DHT[10])
  {
    Serial.print(" *** BAD CRC! ***  ");  
  }     
  else
  {
    Serial.print(" CRC is good  ");
  }
  Serial.print("ZONE 2 Temperature ");
  Serial.print(TempCz2);
  Serial.print(" (C)  ");
  Serial.print(TempFz2);
  Serial.print(" (F) ");
  Serial.print("  Humidity ");
  Serial.println(RHz2); 
}

void get_DHT3()
{
  // DHT[0]=байт команды - установить мастером в значение 4 для запроса показаний DHT #3
  //    и при возврате он будет установлен в 1 в случае успеха.
  // 5 байтов информации DHT означают следующее:
  // данные DHT
  // DHT[11] влажность, целая часть
  // DHT[12] влажность, часть после запятой
  // DHT[13] температура, целая часть
  // DHT[14] температура, часть после запятой
  // DHT[15} контрольная сумма – это суммпа всех четырех байтов AND 255
  Wire.beginTransmission(address);
  Wire.write(0); // индекс в буфере
  Wire.write(4); // байт команды (4=прочитать DHT22 #3)
  Wire.endTransmission();
  delay(50);     // ждать выполнения - для начала 50 миллисекунд
  Wire.beginTransmission(address);
  Wire.write(0); // индекс в буфере
  Wire.endTransmission();
  Wire.requestFrom(address, 1);
  // обновить байт статуса
  DHT[0]=Wire.read();
  // теперь получим 5 байтов данных DHT в массив
  Wire.beginTransmission(address);
  Wire.write(11); // индекс в буфере (указывает на данные для DHT #3)
  Wire.endTransmission();
  Wire.requestFrom(address, 5);
  for (int count=11;count<16; count++)
  {
    DHT[count]=Wire.read();
  } 
  // проверить, указывает ли байт команды, что всё прошло хорошо 
  if (DHT[0]!=1){
    Serial.println("*** Read Error ***");
    // далее обрабатываем ошибку, если это необходимо
  }  
  // Влажность
  RHz3=DHT[11];
  RHz3*=256;
  RHz3+=DHT[12];
  RHz3/=10;
  // Температура  
  TempCz3=DHT[13]&0x7f;
  TempCz3*=256;
  TempCz3+=DHT[14];
  TempCz3/=10;
  if(DHT[13]&0x80)
  {
    TempCz3*=-1;
  }
  TempFz3=TempCz3 * 9 / 5 +32; 
  
  // контрольная сумма
  if ((DHT[11]+DHT[12]+DHT[13]+DHT[14] & 255) != DHT[15])
  {
    Serial.print(" *** BAD CRC! ***  ");  
  }
  else
  {
    Serial.print(" CRC is good  ");
  }
  Serial.print("ZONE 3 Temperature ");
  Serial.print(TempCz3);
  Serial.print(" (C)  ");
  Serial.print(TempFz3);
  Serial.print(" (F) ");
  Serial.print("  Humidity ");
  Serial.println(RHz3); 
}

void resetPIC()
{
  Wire.beginTransmission(address);
  Wire.write(0);    // индекс в буфере
  Wire.write(16);   // байт команды (0x10= перезагрузить Pic)
  Wire.endTransmission();
}  

